/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package us.levk.remote.drmaa.client.impl;

import java.net.URL;
import java.util.Map;
import java.util.UUID;

import org.ggf.drmaa.DrmaaException;
import org.ggf.drmaa.JobInfo;

import us.levk.remote.drmaa.common.api.RemoteJobInfo;

/**
 * Concrete implementation of the org.ggf.drmaa.JobInfo interface making remote calls
 * to the server part with actual DRMAA implementation.
 *
 * Actual implementation is based on XMLRPC which does not support a built in notion of
 * instances, as such the communication layer on the remote end consists of a single
 * RemoteJobInfo instance and the actual JobInfo instance id is passed with every call.
 * 
 * @author levk
 *
 */
public class JobInfoImpl implements JobInfo {

  /**
   * Maps URL to RemoteJobInfo instance
   */
  final transient Map<URL, Map<Class<?>, Object>> remote;
  /**
   * Actual id of this instance
   */
  final UUID info;
  /**
   * URL of the server
   */
  final URL url;

  JobInfoImpl (Map<URL, Map<Class<?>, Object>> remote, UUID info, URL url) {
    this.info = info;
    this.url = url;
    this.remote = remote;
  }

  /* (non-Javadoc)
   * @see org.ggf.drmaa.JobInfo#getJobId()
   */
  @Override public String getJobId () throws DrmaaException {
    return ((RemoteJobInfo) remote.get (url).get (RemoteJobInfo.class)).getJobId (info);
  }

  /* (non-Javadoc)
   * @see org.ggf.drmaa.JobInfo#getResourceUsage()
   */
  @SuppressWarnings ("rawtypes") @Override public Map getResourceUsage () throws DrmaaException {
    return ((RemoteJobInfo) remote.get (url).get (RemoteJobInfo.class)).getResourceUsage (info);
  }

  /* (non-Javadoc)
   * @see org.ggf.drmaa.JobInfo#hasExited()
   */
  @Override public boolean hasExited () throws DrmaaException {
    return ((RemoteJobInfo) remote.get (url).get (RemoteJobInfo.class)).hasExited (info);
  }

  /* (non-Javadoc)
   * @see org.ggf.drmaa.JobInfo#getExitStatus()
   */
  @Override public int getExitStatus () throws DrmaaException {
    return ((RemoteJobInfo) remote.get (url).get (RemoteJobInfo.class)).getExitStatus (info);
  }

  /* (non-Javadoc)
   * @see org.ggf.drmaa.JobInfo#hasSignaled()
   */
  @Override public boolean hasSignaled () throws DrmaaException {
    return ((RemoteJobInfo) remote.get (url).get (RemoteJobInfo.class)).hasSignaled (info);
  }

  /* (non-Javadoc)
   * @see org.ggf.drmaa.JobInfo#getTerminatingSignal()
   */
  @Override public String getTerminatingSignal () throws DrmaaException {
    return ((RemoteJobInfo) remote.get (url).get (RemoteJobInfo.class)).getTerminatingSignal (info);
  }

  /* (non-Javadoc)
   * @see org.ggf.drmaa.JobInfo#hasCoreDump()
   */
  @Override public boolean hasCoreDump () throws DrmaaException {
    return ((RemoteJobInfo) remote.get (url).get (RemoteJobInfo.class)).hasCoreDump (info);
  }

  /* (non-Javadoc)
   * @see org.ggf.drmaa.JobInfo#wasAborted()
   */
  @Override public boolean wasAborted () throws DrmaaException {
    return ((RemoteJobInfo) remote.get (url).get (RemoteJobInfo.class)).wasAborted (info);
  }
}
